-----------------------------------------------------------------------------------------
-- Start Activity
-----------------------------------------------------------------------------------------
-- Script is the main part of the mission. You can't generally create a mission without some Lua
-- so get ready to some light to medium programming and add http://lua.org , Cortex Command Wiki
-- and forum search to the favourites in your browser.
--
-- This method\function is the place where the mission starts. Here you define and initialize variables, 
-- create scripted objects (brains, actors etc) and so on. If something goes wrong in this method, then
-- you kicked out to the campaign screen with error message in the console. Some times you need to restart
-- your mission twice, because CC will keep kicking you out complaining about already fixed errors.
-- Notice the name of the function, MissionTutorial_01 part is that Lua class name that you used in mission activities.ini
function MissionTutorial_01:StartActivity()
	-- Always print debugging information to the console but keep in mind that heavy prinintg
	-- causes heavy lag. Comment all the unnecessary prints when you release your mission to the public.
	-- If you notice some heavy unexpected lag, check the console. CC is probably flooding it with error messages.
	print("START! -- MissionTutorial_01:StartActivity()!");
	
	-- Array which stores flags for players that lost their brain.
	self.braindead = {};
	-- Array which stores brains assigned to players. It is used for quick enumeration of 
	-- playable brains to trigger something or show mission marks.
	self.AssignedBrains = {};
	-- This is used to limit the amount of ingame movable objects (MO's). Currently CC goes mad if this number exceeds
	-- 255. To avoid unexpected behaviour you MUST always take care of the number of the movable ID's. We'll talk more
	-- about movable objects and limits later when we'll discuss spawning. Further reading - http://www.datarealms.com/wiki/index.php/Scenes
	self.MOIDLimit = 200;
    
	-- Here we store the amount of gold that player will recieve at start. Note that we not actualyy give player
	-- real gold here
	self.PlayerGold = 0; -- Player starting credits

   	-- Team variables are used just for convenience
   	-- Team 2 is always CPU
   	self.CPUTeam = Activity.TEAM_2;
	self.PlayerTeam = Activity.TEAM_1;
	self.RogueTeam = -1;

	-- Get area with unique name to prevent CC engine from associating your activity with any other
	-- scenes except those for wich you designed your script. CC simply looks at areas defined in 
	-- scene and compares them to areas got in script with SceneMan.Scene:GetArea(). If everything
	-- matches then CC thinks that scene can be used with acitivty script and show scene on map for
	-- selected activity	
	self.Zone = SceneMan.Scene:GetArea("MissionTut_01_Area");
	
   	-- Set up players
    -- Here we create brains for players. Although current CC don't have predefined activities with multiplayer missions
	-- keep in mind that this behaviour may change, so you should treat gameplay as it was a multiplayer game so you'll have
	-- to fix less when CC will support standard multiplayer activities for missions
	for player = 0, self.PlayerCount - 1 do
        -- Check if we already have a brain assigned
        if not self:GetPlayerBrain(player) then
			self.braindead[player] = false;
			-- Create brain here and alter all his neccessary properties
			local brain = CreateAHuman("Brain Robot");
			-- Actor location on the scene. X=0 , Y=0 is the upper left corner of the map. Greater X is to the right.
			-- Greater Y is to the bottom.
			brain.Pos = Vector(300 + player * 30 , 550);
			-- Create weapon and give it to the actor.
			brain:AddInventoryItem(CreateHDFirearm("Auto Pistol" , "Coalition.rte"));
			-- Make brain just stay and do nothing. Further reading - http://www.datarealms.com/wiki/index.php/LuaDocs/Actor
			brain.AIMode = Actor.AIMODE_SENTRY;
			-- Add created brain to the array of assigned actors
			self.AssignedBrains[player] = brain;

            -- Let the spawn into the world, passing ownership
            MovableMan:AddActor(brain);
            -- Set the found brain to be the selected actor at start
            self:SetPlayerBrain(brain, player);
            SceneMan:SetScroll(self:GetPlayerBrain(player).Pos, player);
            -- Set the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
            self:SetObservationTarget(self:GetPlayerBrain(player).Pos, player);
        end
    end
	
    -- Set the team's funds. Here we actually give a player some gold.
    self:SetTeamFunds(self.PlayerGold , self.PlayerTeam);
end
-----------------------------------------------------------------------------------------
-- Pause Activity
-----------------------------------------------------------------------------------------
-- This function is called when mission is paused
function MissionTutorial_01:PauseActivity(pause)
    print("PAUSE! -- MissionTutorial_01:PauseActivity()!");
end
-----------------------------------------------------------------------------------------
-- End Activity
-----------------------------------------------------------------------------------------
-- This function is called after mission has ended
function MissionTutorial_01:EndActivity()
    print("END! -- MissionTutorial_01:EndActivity()!");
end
-----------------------------------------------------------------------------------------
-- Update Activity
-----------------------------------------------------------------------------------------
-- This is the main loop of the mission. This function is called many times at a second and
-- processes all mission events. You should not write heavy code here, just call some specific functions
-- with actual mission code. Nobody can prevent you from writing all mission code here, but at some moment
-- you'll find that you missed some end somewhere, and you have to check 1000 linies of code to find out where,
-- because Lua gives you wring line numbers in error message
function MissionTutorial_01:UpdateActivity()
   	-- Clear all objective markers, they get re-added each frame
   	self:ClearObjectivePoints();

	-- This mission is empty so we anly check for defeat conditions here
	self:DoBrainSelection();
	
	self:YSortObjectivePoints();
end
-----------------------------------------------------------------------------------------
-- Brain selection and gameover conditions check
-----------------------------------------------------------------------------------------
-- Here we check if we stil have a brain to cobtrol our troops. If not - game over
function MissionTutorial_01:DoBrainSelection()
	-- Check if mission is still running to prevent mission failure
	-- if you waste brain after victory or lose it's object after evacuation
	if not (self.ActivityState == Activity.OVER) then
		-- Iterate through all human players
		for player = 0, self.PlayerCount - 1 do
			-- The current player's team
			local team = self:GetTeamOfPlayer(player);
	
			-- Make sure the game is not already ending
			-- Check if any player's brain is dead
			local brain = self:GetPlayerBrain(player);
			-- Notice the MovableMan:IsActor(brain) function call. This is eesntial when you reference
			-- ingame objects especially actors from lua. It checks if onjects still exists. If you reference 
			-- properties of a non-existent actor (actor can die at any moment) the game will simply crash. 
			-- No fancy console errors at all, just a silent crash.
			if not brain or not MovableMan:IsActor(brain) or not brain:HasObjectInGroup("Brains") then
				-- Reset player brain
				self:SetPlayerBrain(nil, player);
				-- Try to find a new unasigned brain this player can use instead, or if his old brain entered a craft
				local newBrain = MovableMan:GetUnassignedBrain(team);
				-- Found new brain actor, assign it and keep on truckin'
				if newBrain and self.braindead[player] == false then
					self:SetPlayerBrain(newBrain, player);
					self:SwitchToActor(newBrain, player, team);
				else
					-- All brains of this team are dead. Show fail reason
					FrameMan:SetScreenText("Commander died!", player, 333, -1, false);
					-- Set the flag that this player is brainless
					self.braindead[player] = true;
					
					-- Now see if all brains of self player's team are dead, and if so, end the game
					-- Show Fail\Win banner to participating teams. 
					self.WinnerTeam = self:OtherTeam(team);
					-- Game over
					ActivityMan:EndActivity();
				end
			else
				-- Update the observation target to the brain, so that if/when it dies, the view flies to it in observation mode
				self:SetObservationTarget(brain.Pos, player)
			end -- 
		end --for
	end --if
end
-----------------------------------------------------------------------------------------
-- That's all folks!
-----------------------------------------------------------------------------------------
